@@ -841,52 +841,80 @@ JsonNode* exp_lstat(const JsonNode* args, GError** err) {
     return stat_map(&st);
 }
 
+struct probe {
+    int fd;
+    gchar *path;
+};
+
 /**
- * Open an unnamed temporary file at the directory dir. Return an open file
- * descriptor or -errno if creating the temporary file failed.
+ * Create a probe file at the directory dir.
+ * Returns 0 on success and -errno if creating the temporary file failed.
  */
-static int open_tempfile(GString *dir, int flags)
+static int create_probe(struct probe *probe, GString *dir, int flags)
 {
     gchar *uuid = NULL;
     gchar *path = NULL;
-    int fd;
+    int err = 0;
 
     uuid = g_uuid_string_random();
     if (uuid == NULL) {
-        fd = -ENOMEM; /* Not documented, guessing. */
+        err = -ENOMEM; /* Not documented, guessing. */
+        g_warning("Failed to create a probe file, 'g_uuid_string_random' "
+                  "failed, error: '%s'", iop_strerror(-err));
         goto out;
     }
 
     path = g_strdup_printf("%s/.prob-%s", dir->str, uuid);
     if (path == NULL) {
-        fd = -ENOMEM; /* Not documented, guessing. */
+        err = -ENOMEM; /* Not documented, guessing. */
+        g_warning("Failed to create a probe file, 'g_strdup_printf' "
+                  "failed, error: '%s'", iop_strerror(-err));
         goto out;
     }
 
-    fd = open(path, flags | O_CREAT | O_EXCL, S_IRUSR | S_IWUSR);
-    if (fd < 0) {
-        fd = -errno;
+    probe->fd = open(path, flags | O_CREAT | O_EXCL, S_IRUSR | S_IWUSR);
+    if (probe->fd < 0) {
+        g_warning("Failed to create a probe file: '%s', error: '%s'",
+                  path, iop_strerror(errno));
+        err = -errno;
         goto out;
     }
 
-    if (unlink(path) != 0) {
-        int saved_errno = errno;
-        close(fd);
-        fd = -saved_errno;
-    }
+    probe->path = path;
 
 out:
     g_free(uuid);
-    g_free(path);
 
-    return fd;
+    if (err != 0)
+        g_free(path);
+
+    return err;
+}
+
+static int delete_probe(struct probe *probe)
+{
+    int err = 0;
+
+    if (probe->fd != -1) {
+        close(probe->fd);
+
+        if (unlink(probe->path) != 0) {
+            g_warning("Failed to delete a probe file: '%s', error: '%s'",
+                      probe->path, iop_strerror(errno));
+            err = -errno;
+        }
+
+        g_free(probe->path);
+    }
+
+    return err;
 }
 
 JsonNode* exp_probe_block_size(const JsonNode* args, GError** err) {
     GError* tmpError = NULL;
     GString* dir = NULL;
+    struct probe probe = {-1};
     int sizes[] = {1, 512, 4096};
-    int fd;
     int rv;
     void *buf = NULL;
     int block_size = -1;
@@ -899,15 +927,17 @@ JsonNode* exp_probe_block_size(const JsonNode* args, GError** err) {
 
     /* O_DSYNC is required to enforce strict direct I/O if Gluster is
      * configured without performance.strict-o-direct. */
-    fd = open_tempfile(dir, O_WRONLY | O_DIRECT | O_DSYNC);
-    if (fd < 0) {
-        set_error_from_errno(err, IOPROCESS_GENERAL_ERROR, -fd);
+    rv = create_probe(&probe, dir, O_WRONLY | O_DIRECT | O_DSYNC);
+    if (rv != 0) {
+        set_error_from_errno(err, IOPROCESS_GENERAL_ERROR, -rv);
         return NULL;
     }
 
     rv = posix_memalign(&buf, 4096, 4096);
     if (rv != 0) {
         /* posix_memalign does not set errno. */
+        g_warning("Failed to allocate 4K aligned memory, error: '%s'",
+                  iop_strerror(rv));
         set_error_from_errno(err, IOPROCESS_GENERAL_ERROR, rv);
         goto out;
     }
@@ -915,16 +945,20 @@ JsonNode* exp_probe_block_size(const JsonNode* args, GError** err) {
     /* Don't leak memory contents. */
     memset(buf, 0, 4096);
 
+    /* Try to write to the file with different block sizes.
+     * Handle expected EINVAL errors. */
     for (int i = 0; i < (int)ARRAY_SIZE(sizes); i++) {
         block_size = sizes[i];
 
         do {
-            rv = pwrite(fd, buf, block_size, 0);
+            rv = pwrite(probe.fd, buf, block_size, 0);
         } while (rv < 0 && errno == EINTR);
 
         if (rv < 0) {
             if (errno != EINVAL) {
                 /* Unexpected error, bail out. */
+                g_warning("Failed to write %d bytes to probe file: '%s', error: '%s'",
+                          block_size, probe.path, iop_strerror(errno));
                 set_error_from_errno(err, IOPROCESS_GENERAL_ERROR, errno);
                 block_size = -1;
                 goto out;
@@ -943,7 +977,11 @@ JsonNode* exp_probe_block_size(const JsonNode* args, GError** err) {
     block_size = -1;
 
 out:
-    close(fd);
+    /*
+     * Delete the probe file. Do not fail the operation if this fail, since
+     * we may have succeeded to probe the block size, or already
+     * failed because of other reasons. */
+    delete_probe(&probe);
     free(buf);
 
     if (block_size == -1)
